package at.ac.uibk.gitsearch.service.gitlab_events;

import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventService.ExtUserProvidedMetadataDTO;
import com.fasterxml.jackson.core.JacksonException;
import java.util.regex.Pattern;
import org.slf4j.LoggerFactory;

public interface MetaDataParser {
    public enum FormatType {
        YAML,
        JSON,
    }

    public enum ParserType {
        VERSION4(new MetaDatav04Parser()),
        VERSION5(new MetaDatav05Parser());

        private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(ParserType.class);

        private static final Pattern METADATA_VERSION_DETECT = Pattern.compile("metadataVersion:\\W*'0.5'");

        private MetaDataParser metaDataParser;

        private ParserType(MetaDataParser metaDataParser) {
            this.metaDataParser = metaDataParser;
        }

        public MetaDataParser getMetaDataParser() {
            return metaDataParser;
        }

        static ParserType getParserFor(String content) {
            if (METADATA_VERSION_DETECT.matcher(content).find()) {
                LOGGER.debug("Metadata v0.5 detected!");
                return VERSION5;
            } else {
                return VERSION4;
            }
        }
    }

    public ExtUserProvidedMetadataDTO parse(String content, FormatType type) throws JacksonException;
}
