package at.ac.uibk.gitsearch.service.gitlab_events;

import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventService.ExtUserProvidedMetadataDTO;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JacksonException;
import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;

public class MetaDatav05Parser implements MetaDataParser {

    /**
     * just a derived class that does support Version 5 json/yaml parsing.
     * Unfortunately this version needs special conversion of "requires" attribute.
     *
     * @author Michael
     *
     */
    public static class ExtFor5UserProvidedMetadataDTO extends ExtUserProvidedMetadataDTO {

        public ExtUserProvidedMetadataDTO toExtUserProvidedMetadataDTO() {
            ExtUserProvidedMetadataDTO copy = new ExtUserProvidedMetadataDTO();
            copy.setAssesses(this.getAssesses());
            copy.setAudience(this.getAudience());
            copy.setContributor(this.getContributor());
            copy.setCreator(this.getCreator());
            copy.setDescription(this.getDescription());
            copy.setDifficulty(this.getDifficulty());
            copy.setEducationalAlignment(this.getEducationalAlignment());
            copy.setEducationalFramework(this.getEducationalFramework());
            copy.setEducationalLevel(this.getEducationalLevel());
            copy.setEducationalUse(this.getEducationalUse());
            copy.setFormat(this.getFormat());
            copy.setIdentifier(this.getIdentifier());
            copy.setInteractivityType(this.getInteractivityType());
            copy.setIsBasedOn(this.getIsBasedOn());
            copy.setImage(this.getImage());
            copy.setKeyword(this.getKeyword());
            copy.setLanguage(this.getLanguage());
            copy.setLearningResourceType(this.getLearningResourceType());
            copy.setLicense(this.getLicense());
            copy.setMetadataVersion(this.getMetadataVersion());
            copy.setProgrammingLanguage(this.getProgrammingLanguage());
            copy.setPublisher(this.getPublisher());
            copy.setRequires(this.getRequires());
            copy.setStatus(this.getStatus());
            copy.setStructure(this.getStructure());
            copy.setSubject(this.getSubject());
            copy.setTeaches(this.getTeaches());
            copy.setTimeRequired(this.getTimeRequired());
            copy.setTitle(this.getTitle());
            copy.setTypicalAgeRange(this.getTypicalAgeRange());
            copy.setVersion(this.getVersion());
            copy.setValid(this.getValid());
            copy.setCollectionContent(this.getCollectionContent());
            copy.setPublicVisibility(this.getPublicVisibility());
            copy.setRepositoryURL(this.getRepositoryURL());
            return copy;
        }

        @Override
        @JsonProperty(value = "hiddenRequires")
        public String[] getRequires() {
            return super.getRequires();
        }

        @JsonProperty(value = "requires")
        public String getRequiresString() {
            String[] requires = getRequires();
            if (requires == null || requires.length == 0) return null;

            return requires[0];
        }

        public void setRequiresString(String requiresString) {
            if (requiresString == null) setRequires(null); else setRequires(new String[] { requiresString });
        }
    }

    protected MetaDatav05Parser() {
        super();
        yamlMapper.findAndRegisterModules();
        jsonMapper.findAndRegisterModules();
    }

    protected ObjectMapper yamlMapper = new ObjectMapper(new YAMLFactory());

    protected ObjectMapper jsonMapper = new ObjectMapper(new JsonFactory());

    @Override
    public ExtUserProvidedMetadataDTO parse(String content, FormatType type) throws JacksonException {
        ObjectMapper mapper = null;

        switch (type) {
            case YAML:
                mapper = yamlMapper;
                break;
            case JSON:
                mapper = jsonMapper;
                break;
            default:
                mapper = yamlMapper;
        }

        return mapper.readValue(content, ExtFor5UserProvidedMetadataDTO.class).toExtUserProvidedMetadataDTO();
    }
}
